@@ -29,12 +29,12 @@
 #include <sys/types.h>
 
 #include <libtracker-common/tracker-dbus.h>
-#include <libtracker-db/tracker-db-dbus.h>
-#include <libtracker-db/tracker-db-interface-sqlite.h>
 
 #include <libtracker-data/tracker-data-manager.h>
 #include <libtracker-data/tracker-data-update.h>
 #include <libtracker-data/tracker-data-query.h>
+#include <libtracker-data/tracker-db-dbus.h>
+#include <libtracker-data/tracker-db-interface-sqlite.h>
 #include <libtracker-data/tracker-sparql-query.h>
 
 #include "tracker-store.h"
@@ -57,9 +57,10 @@ typedef struct {
 	gboolean     update_running;
 	GThreadPool *main_pool;
 	GThreadPool *global_pool;
-	GSList	    *running_tasks;
-	guint	     watchdog_id;
+	GSList      *running_tasks;
+	guint        watchdog_id;
 	guint        max_task_time;
+	gboolean     active;
 } TrackerStorePrivate;
 
 typedef enum {
@@ -74,10 +75,10 @@ typedef struct {
 	TrackerStoreTaskType  type;
 	union {
 		struct {
-			gchar       *query;
-			GThread     *running_thread;
-			GTimer      *timer;
-			gpointer     thread_data;
+			gchar        *query;
+			GCancellable *cancellable;
+			GTimer       *timer;
+			gpointer      thread_data;
 		} query;
 		struct {
 			gchar        *query;
@@ -138,6 +139,9 @@ store_task_free (TrackerStoreTask *task)
 		if (task->data.query.timer) {
 			g_timer_destroy (task->data.query.timer);
 		}
+		if (task->data.query.cancellable) {
+			g_object_unref (task->data.query.cancellable);
+		}
 	} else {
 		g_free (task->data.update.query);
 	}
@@ -280,7 +284,7 @@ task_ready (TrackerStorePrivate *private)
 static void
 check_handler (TrackerStorePrivate *private)
 {
-	if (task_ready (private)) {
+	if (private->active && task_ready (private)) {
 		/* handler should be running */
 		if (!private->have_handler) {
 			start_handler (private);
@@ -289,6 +293,7 @@ check_handler (TrackerStorePrivate *private)
 		/* handler should not be running */
 		if (private->have_handler) {
 			g_source_remove (private->handler);
+			private->have_handler = FALSE;
 		}
 	}
 }
@@ -309,15 +314,15 @@ watchdog_cb (gpointer user_data)
 
 	while (running) {
 		TrackerStoreTask *task;
-		GThread *thread;
+		GCancellable *cancellable;
 
 		task = running->data;
 		running = running->next;
-		thread = task->data.query.running_thread;
+		cancellable = task->data.query.cancellable;
 
-		if (thread && private->max_task_time &&
+		if (cancellable && private->max_task_time &&
 		    g_timer_elapsed (task->data.query.timer, NULL) > private->max_task_time) {
-			tracker_data_manager_interrupt_thread (task->data.query.running_thread);
+			g_cancellable_cancel (cancellable);
 		}
 	}
 
@@ -353,6 +358,10 @@ task_finish_cb (gpointer data)
 	task = data;
 
 	if (task->type == TRACKER_STORE_TASK_TYPE_QUERY) {
+		if (!task->error) {
+			g_cancellable_set_error_if_cancelled (task->data.query.cancellable, &task->error);
+		}
+
 		if (task->callback.query.query_callback) {
 			task->callback.query.query_callback (task->data.query.thread_data, task->error, task->user_data);
 		}
@@ -361,6 +370,7 @@ task_finish_cb (gpointer data)
 			g_clear_error (&task->error);
 		}
 
+		task->data.query.cancellable = g_cancellable_new ();
 		private->running_tasks = g_slist_remove (private->running_tasks, task);
 		check_running_tasks_watchdog (private);
 		private->n_queries_running--;
@@ -453,7 +463,6 @@ pool_dispatch_cb (gpointer data,
 {
 	TrackerStorePrivate *private;
 	TrackerStoreTask *task;
-	GThread *running_thread = g_thread_self ();
 
 #ifdef __USE_GNU
 	/* special task, only ever sent to main pool */
@@ -471,19 +480,15 @@ pool_dispatch_cb (gpointer data,
 	private = user_data;
 	task = data;
 
-	tracker_data_manager_interrupt_thread_reset (running_thread);
-
 	if (task->type == TRACKER_STORE_TASK_TYPE_QUERY) {
 		TrackerDBCursor *cursor;
 
-		task->data.query.running_thread = running_thread;
 		cursor = tracker_data_query_sparql_cursor (task->data.query.query, &task->error);
 
-		task->data.query.thread_data = task->callback.query.in_thread (cursor, task->error, task->user_data);
+		task->data.query.thread_data = task->callback.query.in_thread (cursor, task->data.query.cancellable, task->error, task->user_data);
 
 		if (cursor)
 			g_object_unref (cursor);
-		task->data.query.running_thread = NULL;
 
 	} else if (task->type == TRACKER_STORE_TASK_TYPE_UPDATE) {
 		if (task->data.update.batch) {
@@ -883,9 +888,9 @@ tracker_store_unreg_batches (const gchar *client_id)
 
 		task = running->data;
 
-		if (task->data.query.running_thread &&
-                    g_strcmp0 (task->client_id, client_id) == 0) {
-			tracker_data_manager_interrupt_thread (task->data.query.running_thread);
+		if (task->data.query.cancellable &&
+		    g_strcmp0 (task->client_id, client_id) == 0) {
+			g_cancellable_cancel (task->data.query.cancellable);
 		}
 	}
 
@@ -933,3 +938,14 @@ tracker_store_unreg_batches (const gchar *client_id)
 
 	check_handler (private);
 }
+
+void
+tracker_store_set_active (gboolean active)
+{
+	TrackerStorePrivate *private;
+
+	private = g_static_private_get (&private_key);
+	private->active = active;
+
+	check_handler (private);
+}
